from cs_ai_common.dynamodb.resolver_task_table import update_resolver_task
from cs_ai_common.dynamodb.task_table import update_task
from cs_ai_common.models.resolvers import ResolverStatsModel, TaskStatisticsModel


def save_task_statistics(task_id: str, stats: TaskStatisticsModel) -> None:
    update_task(task_id, Stats=stats.dict())


def save_resolver_statistics(task_id: str, stats: ResolverStatsModel) -> None:
    update_resolver_task(task_id, stats.ResolverName, found=stats.TotalAds)


import hashlib


def build_cache_key(*args) -> str:
    return hashlib.md5(str(args).lower().encode()).hexdigest()

def build_s3_cache_key(hashed_key: str, resolver_name: str) -> str:
    return f"{hashed_key}/{resolver_name}/result.json"


from typing import Optional
from pydantic import BaseModel

class Filter(BaseModel):
    make: str
    model: str
    min_price: Optional[int] = None
    max_price: Optional[int] = None
    min_year: Optional[int] = None
    max_year: Optional[int] = None
    min_mileage: Optional[int] = None
    max_mileage: Optional[int] = None
    min_engine_capacity: Optional[int] = None
    max_engine_capacity: Optional[int] = None
    min_horsepower: Optional[int] = None
    max_horsepower: Optional[int] = None
    fuel_type: Optional[str] = None
    transmission: Optional[str] = None
    source: Optional[str] = None
    location: Optional[str] = None



from typing import List, Optional
from pydantic import BaseModel

class ResolverResponse(BaseModel): 
    Price: Optional[str] = None
    PriceCurrency: Optional[str] = None
    Mileage: Optional[str] = None
    ProductionYear: Optional[str] = None
    FuelType: Optional[str] = None
    Transmision: Optional[str] = None
    HorsePower: Optional[str] = None
    Capacity: Optional[str] = None
    AdvertisementLink: Optional[str] = None
    Thumbnails: Optional[List[str]] = None
    Source: Optional[str] = None
    SourceId: Optional[str] = None
    Location_Country: Optional[str] = None
    Location_City: Optional[str] = None
    Location_PostalCode: Optional[str] = None
    Make: Optional[str] = None
    Model: Optional[str] = None

class ResolverStatsModel(BaseModel):
    ResolverName: str
    TotalAds: int

class TaskStatisticsModel(BaseModel):
    Stats: List[ResolverStatsModel] = None


import os
from typing import Any
import sqlalchemy

class RdsDbConnection:
    _host: str
    _password: str
    _username: str
    _db_name: str
    _db_port: int
    _engine: sqlalchemy.Engine

    def __init__(self, create_engine: bool = True):
        self.load_config_from_env()

        if create_engine:
            self.create_engine()
    
    def create_engine(self):
        db_schema = "public"
        self._engine = sqlalchemy.create_engine(
            self._create_url(),
            connect_args={'options': '-csearch_path={}'.format(db_schema)})

    def load_config_from_env(self):
        self._host = os.getenv("DB_HOST")
        self._password = os.getenv("DB_PASSWORD")
        self._username = os.getenv("DB_USERNAME")
        self._db_name = os.getenv("DB_NAME")
        self._db_port = os.getenv("DB_PORT")
    
    def get_engine(self) -> sqlalchemy.Engine:
        return self._engine

    def _create_url(self) -> str:
        return f"postgresql://{self._username}:{self._password}@{self._host}:{self._db_port}/{self._db_name}"